import java.awt.*;
import java.awt.geom.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.*;
import java.util.*;
import java.awt.image.BufferedImage;

/** this is a utility class that lets you easily take a 
 * completed animation sequence and push it out to a Quicktime file. 
 * Each frame can take as long as it need to render its image.
 */
 
public class SequenceExporter implements BufferedImageSource{
        protected ImageSource imageSource;
        protected float frameRate;
        protected long start;
        protected long end;
        protected long increment;
        protected boolean isComplete;
        protected BufferedImage bufferedImg1;
        protected Graphics2D graphicsHandle1;
        protected long lastTime =  0;

        protected SequenceExportEventListener el = null;
        
        public SequenceExporter(ImageSource is, long start, long end, long increment){
            imageSource = is;
            this.start = start;
            this.end = end;
            this.increment = increment;
            frameRate = 1000.0f/(float)increment;
                
            bufferedImg1 = new BufferedImage(imageSource.getWidth(),
                    imageSource.getHeight(),BufferedImage.TYPE_INT_RGB );
            graphicsHandle1 = bufferedImg1.createGraphics();
        }
        
        /** set if you want to be able to respond to the exporter events such as a progress bar
         *@param e an export event listener
         */
        public void setEventListener(SequenceExportEventListener e){
            el = e;
        }
        
        public boolean endOfMedia() {
                return isComplete;
        }

        public int getWidth(){
                return imageSource.getWidth();
        }

        public int getHeight(){
                return imageSource.getHeight();
        }

        public float getFrameRate(){
                return frameRate;
        }
        
        public BufferedImage read(long frameNumber){
            long time = start+frameNumber*increment;
            if(time >= end)
                isComplete = true;

            graphicsHandle1.drawImage(imageSource.read(time), 0, 0, null);

            lastTime = time;
            if(el != null)
                el.onExportNewFrame(lastTime);
            return bufferedImg1;
        }

        public void export(String filename,float quality) {
            if(filename.endsWith(".mov")){
                isComplete = false;
                //System.err.println("Starting------------------------");
                if(el != null)
                    el.onExportBegin();
                BufferedImageToQTEncoder encoder = new BufferedImageToQTEncoder(filename, this, quality);
                encoder.run();
                if(el != null)
                    el.onExportFinish();
                //System.err.println("Done------------------------");
            }
            else
                System.out.println("File name must end in .mov!!!!");
        }
}